package fmi19.block1;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.jgrapht.Graph;
import org.jgrapht.graph.DefaultEdge;

/**
 * 
 * A coloring of a graph.
 *
 */
public class Coloring {
	
	/**
	 * The possible colors
	 *
	 */
	public enum Color {
	    RED, GREEN, BLUE, ORANGE
	}
		
	/**
	 * The actual mapping of vertices to colors 
	 */
	private Map<Integer,Color> coloring;

	public Coloring() {
		this.coloring=new HashMap<Integer, Color>();		
	}
	
	public Coloring(Graph<Integer, DefaultEdge> graph) {
		this.coloring=new HashMap<Integer, Color>(graph.vertexSet().size());		
	}
	
	/**
	 * Generates a new coloring equal to an existing one
	 * @param coloring
	 */
	public Coloring(Coloring coloring) {
		this.coloring=new HashMap<Integer, Color>(coloring.getMap());		
	}

	private Map<Integer, Color> getMap() {
		return this.coloring;
	}

	/**
	 * Colors the given vertex with the given color. Overwrites existing colors.
	 * @param vertex
	 * @param color
	 */
	public void setColor(Integer vertex, Color color) {
		coloring.put(vertex,color);
	}
	
	/**
	 * 
	 * @param vertex
	 * @return Returns the color of an vertex
	 */
	public Color getColor(Integer vertex) {
		return coloring.get(vertex);
	}

	/**
	 * removes a vertex from the coloring
	 * @param vertex
	 */
	public void remove (Integer vertex) {
		coloring.remove(vertex);
	}
	
	/**
	 * 
	 * @param vertex
	 * @return true if the vertex is already colored
	 */
	public boolean contains(Integer vertex) {
		return coloring.containsKey(vertex);
	}
		
	public String toString () {
		if (coloring.size()<30) return coloring.toString();
		int red=0,green=0,blue=0,orange=0;
		for(Integer vertex: coloring.keySet()) {
			switch(coloring.get(vertex)) {
				case RED:
					red++;
					break;
				case GREEN:
					green++;
					break;
				case BLUE:
					blue++;
					break;
				case ORANGE:
					orange++;
					break;
			}
		}		
		return coloring.size()+" colored vertices: RED: " +red + ", GREEN: " + green + ", BLUE: " + blue + ", ORANGE: " + orange;
	}
	
	/**
	 * Test whether the stored coloring is valid for the given graph and all vertices in the set redVertices a re colored red 
	 * @param graph
	 * @param redVertices
	 * @return true if the coloring is valid for the given instance
	 */
	public boolean isValidColoringOf(Graph<Integer,DefaultEdge> graph, List<Integer> redVertices) {
		for(Integer vertex: redVertices) {
			if(this.getColor(vertex)!=Color.RED) return false;
		}
		
		return isValidColoringOf(graph);
	}
	
	/**
	 * Test whether the stored coloring is valid for the given graph
	 * 
	 * @param graph
	 * @return true if the coloring is valid for the given instance
	 */
	public boolean isValidColoringOf(Graph<Integer,DefaultEdge> graph) {		
		// TODO
	}	
	
	/**
	 * Brute-force approach to compute the valid colorings of a graph.
	 * It enumerates all colorings and test validity by the isValidColoringOf(graph) method.
	 * 
	 * @param graph
	 * @param number upper bound for the number of valid colorings to compute
	 * @return A List of valid colorings for the given graph
	 */
	public static List<Coloring> computeColorings(Graph<Integer,DefaultEdge> graph, int number) {

		/* This optional and will NOT be graded!!!
		 * 
		 * You an implement a simple graph algorithm that computes the valid colorings (or up to number many colorings)
		 * and compare the performance with you SAT-Encoding  
		 * 
		 */
		return null;		
	}
}
